/*
	ReverseFieldDominance plugin for Avisynth -- lets you reverse the
	field dominance of a clip.

  This is particularly useful for changing PAL DV from bottom field
  first to top field first.

	Copyright (C) 2002-2003 Simon Walters, All Rights Reserved

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Simon Walters
	siwalters@hotmail.com

	V1.0 - 1st release.
	V1.1 - shift down as well as up
	V1.2 - handles RGB Input Correctly = Thanks to Wilbert for poiinting out error
*/
#include "windows.h"
#include "avisynth.h"
//#include "internal.h"

#include "memory.h"



class ReverseFieldDominance : public GenericVideoFilter {
	bool ShiftUp;
	int Param2, Param3, Param4;

public:
	ReverseFieldDominance(PClip _child,
		bool _ShiftUp, int _Param2, int _Param3,int _Param4,
		IScriptEnvironment* env) :
	GenericVideoFilter(_child),
		ShiftUp(_ShiftUp), Param2(_Param2),
		 Param3(_Param3), Param4(_Param4)

	{
	// parameter checking goes in here
	if (vi.IsYV12())
			env->ThrowError("ReverseFieldDominance: input to filter must be in YUY2 or RGB");
	}

    PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
};



PVideoFrame __stdcall ReverseFieldDominance::GetFrame(int n, IScriptEnvironment* env) {
	
	
	PVideoFrame src = child->GetFrame(n, env);
	PVideoFrame dst = env->NewVideoFrame(vi);
	
	const unsigned char* srcp = src->GetReadPtr();
	unsigned char* dstp = dst->GetWritePtr();
	
	
	const int dst_pitch = dst->GetPitch();
	const int dst_width = dst->GetRowSize();
	const int dst_height = dst->GetHeight();
	const int src_pitch = src->GetPitch();
	const int src_width = src->GetRowSize();
	const int src_height = src->GetHeight();
	
	int dst_modulo;
	dst_modulo = dst_pitch - dst_width;
	
	
	int h;
	
	//Copy src to dst
	
	
	if ((ShiftUp && !vi.IsRGB()) || (!ShiftUp && vi.IsRGB()) ) {
		
		srcp = src->GetReadPtr();
		dstp = dst->GetWritePtr();		
		srcp = srcp + src_pitch;
		
		for (h = 0; h < src_height - 1; h++)
		{
			memcpy(dstp, srcp, src_width);
			srcp += src_pitch;
			dstp += dst_pitch;
		}
		memcpy(dstp, srcp - src_pitch, src_width);
	}
	else
	{
		srcp = src->GetReadPtr();
		dstp = dst->GetWritePtr();
		
		memcpy(dstp, srcp, src_width);
		
		dstp = dstp + dst_pitch;
		
		for (h = 0; h < src_height - 1; h++)
		{
			memcpy(dstp, srcp, src_width);
			srcp += src_pitch;
			dstp += dst_pitch;
		}
	}
	return dst;
	
	
	
}


AVSValue __cdecl Create_ReverseFieldDominance(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new ReverseFieldDominance(args[0].AsClip(),
		 args[1].AsBool(true),	// 
		 args[2].AsInt(0),	// 
		 args[3].AsInt(0),	// 
		 args[4].AsInt(0),	//

		 env);
}


extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("ReverseFieldDominance", "c[shiftup]b[PARAM2]i[PARAM3]i[PARAM4]i",
		 Create_ReverseFieldDominance, 0);
    return "`ReverseFieldDominance' ReverseFieldDominance plugin";
}

